import React, { useState, useRef, useEffect } from 'react'
import { Button, Popconfirm, Table, message, Tabs } from 'antd'
import { ModalForm, ProFormText, ProFormSelect } from '@ant-design/pro-form'
import ShowTitleMenu from '../Attendance/components/ShowTitleMenu'
import type { ColumnsType } from 'antd/lib/table'
import { MenuOutlined } from '@ant-design/icons'
import { Delete } from '@icon-park/react'
import { SortableContainer, SortableElement, SortableHandle } from 'react-sortable-hoc'
import { arrayMoveImmutable } from 'array-move'
import {
  fetchInvoiceList,
  addInvoiceItem,
  updateInvoiceItem,
  deleteInvoiceItem,
  sortInvoiceItem
} from '@/services/user/system'
import { useRequest } from 'umi'
import classNames from 'classnames'
import './index.less'
import { PageContainer } from '@ant-design/pro-layout'

const DragHandle = SortableHandle(() => <MenuOutlined style={{ cursor: 'grab', color: '#999' }} />)
const SortableItem = SortableElement(props => <tr {...props} />)
const SortableBody = SortableContainer(props => <tbody {...props} />)
const isEnableEnum = {
  open: '启用',
  close: '停用'
}
enum modalType {
  CREATE = 0,
  UPDATE = 1
}
const modalTitleType = {
  1: '开票内容'
}

const Invoice: React.FC = () => {
  const formRef = useRef<FormInstance>(null)
  const [state, setState] = useState({
    id: '',
    activeKey: '6',
    menuId: 1,
    dataSource: [],
    visible: false,
    enable: 'open',
    type: modalType.CREATE
  })
  const { TabPane } = Tabs
  const onSelect = (menuId: string) => {
    setState({ ...state, menuId })
  }
  const { run: tryGetInvoiceList } = useRequest(() => fetchInvoiceList({ rate: state.activeKey }), {
    onSuccess: (result: API.InvoiceList[]) => {
      setState({ ...state, dataSource: result.map((item, index) => ({ ...item, index })) })
    }
  })
  const { run: tryAddInvoiceItem } = useRequest(addInvoiceItem, {
    manual: true,
    onSuccess: async () => {
      tryGetInvoiceList()
      formRef.current?.resetFields()
      message.success('添加成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })
  const { run: tryUpdateInvoiceItem } = useRequest(updateInvoiceItem, {
    manual: true,
    onSuccess: async () => {
      tryGetInvoiceList()
      formRef.current?.resetFields()
      message.success('编辑成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })
  const { run: tryDeleteInvoiceItem } = useRequest(deleteInvoiceItem, {
    manual: true,
    onSuccess: async () => {
      await tryGetInvoiceList()
    },
    onError: e => {
      message.error(e.message)
    }
  })
  const { run: trySortInvoiceItem } = useRequest(sortInvoiceItem, {
    manual: true,
    onSuccess: async () => {
      await tryGetInvoiceList()
      message.success('更新成功')
    },
    onError: e => {
      message.error(e.message)
    }
  })
  useEffect(() => {
    tryGetInvoiceList(state.activeKey)
  }, [state.activeKey])
  const mainColumns: ColumnsType<API.InvoiceList> = [
    {
      dataIndex: 'sort',
      title: '排序',
      render: () => <DragHandle />
    },
    {
      dataIndex: 'name',
      title: '开票内容'
    },
    {
      dataIndex: 'enable',
      title: '状态',
      render: enable => (
        <span className={classNames({ 'text-red-500': enable === 'close' })}>
          {isEnableEnum[enable]}
        </span>
      )
    },
    {
      dataIndex: 'opreate',
      title: '操作',
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row">
          <div
            className="pr-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              formRef.current?.setFieldsValue({ ...record })
              setState({ ...state, visible: true, type: modalType.UPDATE })
            }}>
            编辑
          </div>
          {record.enable === 'open' ? (
            <Popconfirm
              title="是否停用此开票内容"
              onConfirm={() =>
                tryUpdateInvoiceItem({
                  id: record.id,
                  name: record.name,
                  sort: record.sort,
                  enable: 'close'
                })
              }>
              <div className="px-2 text-red-500 cursor-pointer hover:text-red-600">停用</div>
            </Popconfirm>
          ) : null}
          {record.enable === 'close' || record.enable === '' ? (
            <Popconfirm
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              title="是否启用此开票内容"
              onConfirm={() =>
                tryUpdateInvoiceItem({
                  id: record.id,
                  name: record.name,
                  sort: record.sort,
                  enable: 'open'
                })
              }>
              启用
            </Popconfirm>
          ) : null}
          {/* {record.enable === 'close' && (
            <Popconfirm
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              title="是否启用此开票内容"
              onConfirm={() =>
                tryUpdateInvoiceItem({
                  id: record.id,
                  name: record.name,
                  sort: record.sort,
                  enable: 'open'
                })
              }>
              启用
            </Popconfirm>
          )} */}
          <Popconfirm title="确认删除？" onConfirm={() => tryDeleteInvoiceItem(record.id)}>
            <div className="pl-2 text-hex-fd3995 cursor-pointer hover:text-hex-e7026e">
              <Delete />
            </div>
          </Popconfirm>
        </div>
      )
    }
  ]

  const onSortEnd = ({ oldIndex, newIndex }) => {
    const { dataSource } = state
    if (oldIndex !== newIndex) {
      const newData = arrayMoveImmutable([].concat(dataSource), oldIndex, newIndex).filter(
        el => !!el
      )
      // console.log('Sorted items: ', newData)
      setState({ ...state, dataSource: newData })
      const oldIndexItem = dataSource?.find(item => item.index === oldIndex)
      const newIndexItem = dataSource?.find(item => item.index === newIndex)
      if (oldIndexItem && newIndexItem) {
        trySortInvoiceItem({ id: oldIndexItem.id, sortId: newIndexItem.id })
      }
    }
  }

  const DraggableContainer = props => (
    <SortableBody
      useDragHandle
      disableAutoscroll
      helperClass="row-dragging"
      onSortEnd={onSortEnd}
      {...props}
    />
  )

  const DraggableBodyRow = ({ ...restProps }) => {
    const { dataSource } = state
    const index = dataSource?.findIndex(x => x.index === restProps['data-row-key'])
    return <SortableItem index={index} {...restProps} />
  }

  return (
    <PageContainer title={false} breadcrumb={false}>
      <div className="h-full w-full flex flex-row">
        <ShowTitleMenu
          onSelect={onSelect}
          options={[{ label: '开票内容', value: 1 }]}
          defaultValue={1}
        />
        <div className="w-max-3/4">
          <div className="ml-8 bg-white p-4 shadow-md shadow-hex-3e2c5a relative">
            <div className="absolute right-4 top-4 z-100">
              <Button
                type="primary"
                onClick={() => setState({ ...state, visible: true, type: modalType.CREATE })}>
                添加新内容
              </Button>
            </div>
            {state.menuId === 1 ? (
              <Tabs
                defaultActiveKey="6"
                type="card"
                onChange={key => setState({ ...state, activeKey: key })}>
                <TabPane tab="开票内容 1" key="6">
                  <Table<API.InvoiceList>
                    pagination={false}
                    columns={mainColumns}
                    dataSource={state.dataSource}
                    components={{
                      body: {
                        wrapper: DraggableContainer,
                        row: DraggableBodyRow
                      }
                    }}
                    bordered
                    rowKey="index"
                  />
                </TabPane>
                <TabPane tab="开票内容 2" key="13">
                  <Table<API.InvoiceList>
                    pagination={false}
                    columns={mainColumns}
                    dataSource={state.dataSource}
                    components={{
                      body: {
                        wrapper: DraggableContainer,
                        row: DraggableBodyRow
                      }
                    }}
                    bordered
                    rowKey="index"
                  />
                </TabPane>
              </Tabs>
            ) : // <>
            //   <div className="pb-3 border-b border-b-black border-opacity-8 mb-3 flex items-center justify-between">
            //     <div>开票内容</div>
            //     <Button
            //       type="primary"
            //       onClick={() => setState({ ...state, visible: true, type: modalType.CREATE })}>
            //       添加新内容
            //     </Button>
            //   </div>
            //   <Table
            //     pagination={false}
            //     columns={mainColumns}
            //     dataSource={state.dataSource}
            //     components={{
            //       body: {
            //         wrapper: DraggableContainer,
            //         row: DraggableBodyRow
            //       }
            //     }}
            //     bordered
            //     rowKey="index"
            //   />
            // </>
            null}
          </div>
        </div>
        <ModalForm
          visible={state.visible}
          formRef={formRef}
          onVisibleChange={show => setState({ ...state, visible: show })}
          title={`${state.type === modalType.CREATE ? '新增' : '编辑'}${
            modalTitleType[state.menuId]
          }`}
          onFinish={async value => {
            try {
              if (state.type === modalType.CREATE && state.menuId === 1) {
                await tryAddInvoiceItem({
                  ...value
                })
              }
              if (state.type === modalType.UPDATE && state.menuId === 1) {
                await tryUpdateInvoiceItem({
                  ...value
                })
              }
              return true
            } catch (error) {
              message.error(`${state.type === modalType.CREATE ? '创建' : '更新'}失败，请重试`)
              return false
            }
          }}>
          {state.menuId === 1 ? (
            <>
              <ProFormText name="id" hidden />
              <ProFormText name="name" label="开票内容" required />
              <ProFormSelect
                options={[
                  {
                    value: '6',
                    label: '6'
                  },
                  {
                    value: '13',
                    label: '13'
                  }
                ]}
                name="rate"
                label="税率"
              />
            </>
          ) : null}
        </ModalForm>
      </div>
    </PageContainer>
  )
}

export default Invoice
