import React, { useState, useEffect, useRef } from 'react'
import { PlusOutlined, InfoCircleOutlined, FormOutlined, DeleteOutlined } from '@ant-design/icons'
import { Button, message, Popconfirm, Switch, Tooltip } from 'antd'
import { useModel, useRequest } from 'umi'
import {
  createRoleWithMenuId,
  fetchRoleListByMenuId,
  deleteRole,
  updateStaff
} from '@/services/user/api'
import { ProFormDependency, ProFormSelect } from '@ant-design/pro-form'
import { ModalForm, ProFormText } from '@ant-design/pro-form'
import classNames from 'classnames'
import type { ProFormInstance } from '@ant-design/pro-form'
import styles from './index.less'
import consts from '@/utils/consts'
type RoleMenuProps = {
  menuId: string
  onSelect?: (id: string) => void
  itemCount?: number
}
enum ModalType {
  ADD,
  UPDATE
}
type RoleMenuState = {
  menuRoles: API.MenuRoleItem[]
  activeId: Nullable<string>
  visible: boolean
  modalType: ModalType
  defaultMenuData: Nullable<{ id: string; name: string }>
  showExtraFormItem: boolean
}
const RoleMenu: React.FC<RoleMenuProps> = ({ menuId, onSelect, itemCount }) => {
  const formRef = useRef<ProFormInstance>(null)
  const { initialState } = useModel('@@initialState')
  const [state, setState] = useState<RoleMenuState>({
    menuRoles: [],
    activeId: null,
    visible: false,
    modalType: ModalType.ADD,
    showExtraFormItem: false, // 展示额外的表单项
    defaultMenuData: null
  })

  useEffect(() => {
    formRef.current?.setFieldsValue({ ...state.defaultMenuData })
  }, [state.defaultMenuData])

  const { run: tryFetchMenuRoles } = useRequest((id: string) => fetchRoleListByMenuId(id), {
    manual: true,
    onSuccess: result => {
      setState({ ...state, menuRoles: result })
    }
  })

  const { run: tryDeleteRole } = useRequest((id: string) => deleteRole({ id }), {
    manual: true,
    onSuccess: () => {
      tryFetchMenuRoles(menuId)
    },
    onError: e => {
      message.error(e.message)
    }
  })
  const { run: tryUpdateStaff } = useRequest(updateStaff, {
    manual: true,
    onSuccess: () => {
      message.success('修改成功')
      tryFetchMenuRoles(menuId)
    }
  })

  const { run: tryAddRole } = useRequest(
    (params: API.CreateRoleParams) => createRoleWithMenuId(params),
    {
      manual: true,
      onSuccess: () => {
        tryFetchMenuRoles(menuId)
        message.success('添加成功')
      }
    }
  )
  useEffect(() => {
    tryFetchMenuRoles(menuId)
  }, [])

  const handleItemClick = (id: string) => {
    setState({ ...state, activeId: id })
    if (onSelect) {
      onSelect(id)
    }
  }

  const handleEditIconClick = (id: string, text: string) => {
    setState({
      ...state,
      visible: true,
      modalType: ModalType.UPDATE,
      defaultMenuData: { id, name: text }
    })
  }
  return (
    <div
      className="w-max-234px rounded-4px"
      style={{
        height: `calc(100vh - 96px)`
      }}>
      <div className="p-4 border-b-1 border-solid border-black border-opacity-10 bg-hex-f7f9fa flex items-center justify-around">
        <span className="text-[0.9375rem]">角色列表</span>
        <Button
          size="small"
          type="primary"
          ghost
          onClick={() =>
            setState({ ...state, visible: true, modalType: ModalType.ADD, defaultMenuData: null })
          }>
          <PlusOutlined />
          创建角色
        </Button>
      </div>
      <div className="p-4 bg-white" style={{ height: 'calc(100% - 1rem*2 - 20px)' }}>
        <ul className="p-0 m-0 list-none text-primary flex flex-col flex-1">
          {state.menuRoles.map(item => (
            <li
              key={item.id}
              className={classNames(
                'flex justify-between items-center py-2 px-5 cursor-pointer',
                {
                  'scale-up-center': item.id === state.activeId
                },
                styles.node
              )}>
              <div className={styles.title} onClick={() => handleItemClick(item.id)}>
                {item.name}
              </div>
              <div className={styles.extra}>
                <FormOutlined
                  className="pr-2"
                  onClick={() => handleEditIconClick(item.id, item.name)}
                />
                <Popconfirm
                  title="确认删除吗？"
                  okText="确认"
                  cancelText="取消"
                  onConfirm={() => {
                    if (itemCount && itemCount !== 0) {
                      return message.warning('请先移除已经关联的员工')
                    }
                    return tryDeleteRole(item.id)
                  }}>
                  <DeleteOutlined />
                </Popconfirm>
              </div>
            </li>
          ))}
        </ul>
      </div>
      <ModalForm<{ name: string; backstageMenuId: string }>
        visible={state.visible}
        formRef={formRef}
        isKeyPressSubmit
        onVisibleChange={visible => {
          if (!visible) {
            setState({ ...state, visible, showExtraFormItem: false, defaultMenuData: null })
          } else {
            setState({ ...state, visible })
          }
        }}
        title={state.modalType === ModalType.ADD ? '添加新的角色' : '更新角色'}
        width="500px"
        layout="horizontal"
        onFinish={async values => {
          if (state.modalType === ModalType.ADD) {
            await tryAddRole(values)
          } else {
            await tryUpdateStaff(values)
          }
          return true
        }}
        initialValues={{ backstageMenuId: menuId }}>
        <ProFormText name="backstageMenuId" hidden />
        <ProFormText name="id" hidden />
        <ProFormText
          name="name"
          label="名称"
          placeholder="请输入角色名称"
          rules={[{ required: true, message: '请输入角色名' }]}
        />
        {state.modalType === ModalType.ADD ? (
          <div className="mb-6">
            <Switch onChange={checked => setState({ ...state, showExtraFormItem: checked })} />
            <span className="mx-2">复制角色</span>
            <Tooltip title="该操作允许复制其他栏目的角色并且将所绑定的用户一并复制">
              <InfoCircleOutlined />
            </Tooltip>
          </div>
        ) : null}
        {state.showExtraFormItem ? (
          <>
            <ProFormSelect
              label="菜单"
              name="copyMenuId"
              required
              options={initialState?.menuList
                ?.filter(item => item.id !== menuId)
                .map(item => ({ label: item.name, value: item.id }))}
            />
            <ProFormDependency name={['copyMenuId']}>
              {({ copyMenuId }) => (
                <ProFormSelect
                  name="roleId"
                  label="角色"
                  required
                  params={copyMenuId}
                  request={async () => {
                    if (!copyMenuId) return []
                    const { code = -1, data = [] } = await fetchRoleListByMenuId(copyMenuId)
                    if (code === consts.RET_CODE.SUCCESS) {
                      return data.map(item => ({ label: item.name, value: item.id }))
                    }
                    return []
                  }}
                />
              )}
            </ProFormDependency>
          </>
        ) : null}
      </ModalForm>
    </div>
  )
}

export default RoleMenu
